import 'package:flutter/material.dart';

class InputLocation {
  /// 起始位置
  final int start;

  /// 下划线长度
  final int end;

  InputLocation(this.start, this.end);

  @override
  String toString() {
    return 'InputLocation {start: $start, end: $end}';
  }
}

class AutoJumpIntent extends Intent {
  /// 处理自动切换到下一个输入位置的逻辑
  void handleAutoJump(TextEditingController controller) {
    final String text = controller.text;
    final int currBaseOffset = controller.selection.baseOffset;
    final int currExtentOffset = controller.selection.extentOffset;
    final List<InputLocation> locations = findInputLocation(text);
    if (locations.isEmpty) {
      return;
    }

    /// 查找下一个位置终点，如果没有找到，就转到第一个，如果已经选中，寻找再下一个位置
    for (int i = 0; i < locations.length; i++) {
      final InputLocation location = locations[i];
      if (location.end > currBaseOffset) {
        final nextBaseOffset = location.start;
        final nextExtentOffset = location.end;
        if (currBaseOffset == nextBaseOffset &&
            currExtentOffset == nextExtentOffset) {
          continue;
        }
        controller.selection = TextSelection(
            baseOffset: location.start, extentOffset: location.end);
        return;
      }
    }
    controller.selection = TextSelection(
        baseOffset: locations.first.start, extentOffset: locations.first.end);
  }

  /// 查找文本中连续的 _ 符号，作为每一个输入位置
  List<InputLocation> findInputLocation(String text) {
    final List<InputLocation> inputPosition = [];

    /// 查找连续的 _ 符号
    final RegExp regExp = RegExp(r'(_+)', multiLine: true);
    final Iterable<RegExpMatch> matches = regExp.allMatches(text);
    for (final RegExpMatch match in matches) {
      final int start = match.start;
      final int end = match.end;
      inputPosition.add(InputLocation(start, end));
    }
    return inputPosition;
  }
}
