import 'package:fis_lib_report/converts/alignment_convert.dart';
import 'package:fis_lib_report/converts/margin_convert.dart';
import 'package:fis_lib_report/converts/pt_to_px_converter.dart';
import 'package:fis_lib_report/report/input_image.dart';
import 'package:fis_lib_report/report_info/element_tag_names.dart';
import 'package:fis_lib_report/report_info/input_image_info.dart';
import 'package:fis_lib_report/report_info/report_info.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:fis_lib_business_components/index.dart';
import 'package:fis_jsonrpc/rpc.dart';

class FInputImage extends StatefulWidget implements FWidget {
  final InputImage inputImage;

  const FInputImage(this.inputImage, {Key? key}) : super(key: key);
  @override
  State<StatefulWidget> createState() {
    return _FInputImageState();
  }
}

class _FInputImageState extends State<FInputImage> {
  InputImageInfo? _inputImageInfo;
  _FInputImageState();
  Color _borderColor = Colors.grey;
  bool _isSelected = false;
  bool _hasImageBorder = false;
  String _imageUrl = '';
  late String _tagName;
  late Widget _child;

  @override
  initState() {
    _initDatas();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      //  width: width,
      child: MouseRegion(
        cursor: _tagName == TagNames.DIGITALSIGNATURE
            ? SystemMouseCursors.basic
            : SystemMouseCursors.click,
        child: GestureDetector(
          onTap: () {
            if (_tagName == TagNames.DIGITALSIGNATURE) {
              return;
            }
            if (_isSelected) {
              setState(() {
                _borderColor = Colors.grey;
                _isSelected = false;
                _hasImageBorder = false;
                if (_inputImageInfo != null) {
                  _inputImageInfo!.isSelected = false;
                }
              });
            } else {
              setState(() {
                _borderColor = const Color.fromARGB(255, 64, 159, 248);
                _isSelected = true;
                _hasImageBorder = true;
                if (_inputImageInfo != null) {
                  _inputImageInfo!.isSelected = true;
                }
              });
            }
          },
          child: Row(
            mainAxisAlignment: AlignmentConvert.horizontalToMainConvert(
                widget.inputImage.horizontalAlignment),
            children: [
              Container(
                padding: const EdgeInsets.all(3),
                height:
                    PtToPxConverter.ptToPx(widget.inputImage.imageHeight!) + 6,
                // width: width,
                alignment: Alignment.center,
                margin: MarginConvert.marginConvert(widget.inputImage.margin),
                decoration: BoxDecoration(
                    border: _hasImageBorder
                        ? Border.all(
                            width: 0.5,
                            color: _borderColor,
                          )
                        : null,
                    color: Colors.transparent),
                child: _child,
              ),
            ],
          ),
        ),
      ),
    );
  }

  void _initDatas() {
    _child = FText(FReportInfo.instance.selectImageHint);
    final element = FReportInfo.instance.getElementInfo(widget.inputImage);
    if (element != null) {
      final info = element as InputImageInfo;
      if (_inputImageInfo != info) {
        _inputImageInfo = info;
      }
      _tagName = '';
      if (_inputImageInfo != null) {
        if (_inputImageInfo!.tag != null) {
          _tagName = _inputImageInfo!.tag!.name ?? '';
          if (_tagName == TagNames.DIGITALSIGNATURE) {
            _child = const SizedBox();
          }
        }
      }
      _inputImageInfo!.onSelect!.dispose();
      _inputImageInfo!.onSelect!.addListener((sender, e) {
        if (mounted) {
          setState(() {
            if (_imageUrl != e) {
              _imageUrl = e;
              _child = _getImageView();
            } else {
              _imageUrl = '';
              if (_inputImageInfo != null &&
                  _tagName != TagNames.DIGITALSIGNATURE) {
                _child = FText(FReportInfo.instance.selectImageHint);
              }
            }
          });
        }
      });
      _inputImageInfo!.onSelectedChange.dispose();
      _inputImageInfo!.onSelectedChange.addListener((sender, e) {
        if (_tagName == TagNames.DIGITALSIGNATURE) {
          setState(() {
            _imageUrl = e;
            _child = _getSignatureImageView();
          });
          return;
        }
        setState(() {
          if (_imageUrl != e) {
            _imageUrl = e;
            _child = _getImageView();
          } else {
            _imageUrl = '';
            _child = FText(FReportInfo.instance.selectImageHint);
          }
        });
      });
    }
  }

  Widget _getImageView() {
    if (_imageUrl.isNotEmpty) {
      // VidImageViewController;
      return FContentImage(
        terminalImage: TerminalImage(
          coverImageUrl: _imageUrl,
          previewUrl: _imageUrl,
          imageUrl: _imageUrl,
        ),
        onTap: () {
          _inputImageInfo!.selectedImage =
              'http://cdn-bj.fis.plus/db38b8d36a7b48ada3effe0a53ea8993.jpg';
        },
        fileDataType: RemedicalFileDataTypeEnum.Image,
      );
    }
    return FText(FReportInfo.instance.selectImageHint);
  }

  Widget _getSignatureImageView() {
    if (_imageUrl.isNotEmpty) {
      return FFittedBox(
        fit: BoxFit.scaleDown,
        child: FImage.network(
          _imageUrl,
        ),
      );
    }
    return const FSizedBox();
  }
}
