import 'package:fis_lib_report/converts/margin_convert.dart';
import 'package:fis_lib_report/converts/pt_to_px_converter.dart';
import 'package:fis_lib_report/report/cell_postion.dart';
import 'package:fis_lib_report/report/input_image_list.dart';
import 'package:fis_lib_report/report_info/cell_position_info.dart';
import 'package:fis_lib_report/report_info/input_image_list_info.dart';
import 'package:fis_lib_report/report_info/report_info.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:flutter/material.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_lib_business_components/index.dart';

class FInputImageList extends StatefulWidget implements FInteractiveContainer {
  final InputImageList inputImageList;
  @override
  final String pageName = 'FInputImageList';
  final CellPostion? cellPostion;
  final String parentTableId;
  const FInputImageList(this.inputImageList, this.cellPostion,
      {Key? key, this.parentTableId = ""})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _FInputImageListState();
  }
}

class _FInputImageListState extends State<FInputImageList> {
  InputImageList? inputImageList;

  InputImageListInfo? _inputImageListInfo;
  Color _borderColor = Colors.grey;
  bool _isSelected = false;
  List<String> _images = [];
  double _height = 0.0;

  FWidget _child = FText(FReportInfo.instance.selectImageHint);

  @override
  initState() {
    _initDatas();
    super.initState();
  }

  @override
  void didUpdateWidget(covariant FInputImageList oldWidget) {
    super.didUpdateWidget(oldWidget);
    _heightAdaptation();
  }

  @override
  FWidget build(BuildContext context) {
    _checkInputImageListInfo();
    //行数
    final rowCount = (_images.length / inputImageList!.column!).ceil();
    return FSizedBox(
      width: PtToPxConverter.ptToPx(
          inputImageList!.imageWidth! * inputImageList!.column!),
      child: FMouseRegion(
        cursor: SystemMouseCursors.click,
        child: FGestureDetector(
          businessParent: widget,
          name: "selectedInputChange",
          onTap: () {
            _onSelectedInputChange();
          },
          child: FContainer(
            padding: const EdgeInsets.symmetric(vertical: 2, horizontal: 2),
            height: _height + 4 - rowCount,
            width: PtToPxConverter.ptToPx(
                inputImageList!.imageWidth! * inputImageList!.column!),
            alignment: Alignment.center,
            margin: MarginConvert.marginConvert(inputImageList!.margin),
            decoration: BoxDecoration(
                border: _isSelected
                    ? Border.all(
                        width: 0.5,
                        color: _borderColor,
                      )
                    : null,
                color: Colors.transparent),
            child: _child,
          ),
        ),
      ),
    );
  }

  @override
  void dispose() {
    _inputImageListInfo!.onSelect.dispose();
    super.dispose();
  }

  FWidget _getImageListChild() {
    if (_images.isNotEmpty) {
      final imageHeight = PtToPxConverter.ptToPx(inputImageList!.imageHeight!);
      final rowCount = (_images.length / inputImageList!.column!).ceil();
      _height = rowCount * imageHeight + rowCount;
      return FGridView.builder(
        primary: true,
        gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
          crossAxisCount: inputImageList!.column!,
          crossAxisSpacing: 1,
          mainAxisSpacing: 1,
          childAspectRatio:
              inputImageList!.imageWidth! / inputImageList!.imageHeight!,
        ),
        itemCount: _images.length,
        itemBuilder: (BuildContext context, int index) {
          return FContainer(
            height: imageHeight,
            color: Colors.black,
            child: FStack(
              alignment: Alignment.center,
              children: [
                FContentImage(
                  fit: BoxFit.fitHeight,
                  isPureImage: true,
                  fileDataType: RemedicalFileDataTypeEnum.Image,
                  terminalImage: TerminalImage(
                    coverImageUrl: _images[index],
                    previewUrl: _images[index],
                    imageUrl: _images[index],
                  ),
                  onTap: () {
                    _onSelectedInputChange();
                  },
                  ifShowIndex: false,
                ),
                FPositioned(
                  right: 2,
                  top: 2,
                  child: FIconButton(
                    name: "Removeimage",
                    businessParent: widget,
                    icon: const FIcon(
                      Icons.highlight_off,
                      color: Colors.white,
                    ),
                    onPressed: () {
                      var image = _images[index];
                      _inputImageListInfo!.onRemove.emit(this, image);
                      _onSelected(image);
                    },
                  ),
                ),
              ],
            ),
          );
        },
      );
    } else {
      _height = PtToPxConverter.ptToPx(inputImageList!.imageHeight!);
    }

    return FText(FReportInfo.instance.selectImageHint);
  }

  void _addSelectImageListening() {
    try {
      _inputImageListInfo!.onSelectedChange.addListener((sender, e) {
        if (mounted) {
          setState(() {
            _images = e;
            _child = _getImageListChild();
          });
          _heightAdaptation();
        }
      });
    } catch (e) {
      print(e);
    }
  }

  void _addOnSelectedListening() {
    _inputImageListInfo!.onSelect.dispose();
    _inputImageListInfo!.onSelect.addListener((sender, e) => _onSelected(e));
  }

  void _onSelected(String e) {
    if (mounted) {
      setState(() {
        if (_images.contains(e)) {
          _images.remove(e);
          if (_inputImageListInfo!.selectedImages.contains(e)) {
            _inputImageListInfo!.selectedImages.remove(e);
          }
        } else {
          _images.add(e);
          if (!_inputImageListInfo!.selectedImages.contains(e)) {
            _inputImageListInfo!.selectedImages.add(e);
          }
        }
        _child = _getImageListChild();
      });
      _heightAdaptation();
    }
  }

  void _initDatas() {
    inputImageList = widget.inputImageList;
    _height = PtToPxConverter.ptToPx(inputImageList!.imageHeight);
    final element = FReportInfo.instance.getBlockElement(inputImageList!);
    if (element != null) {
      _inputImageListInfo = element as InputImageListInfo;
      //  _addOnSelectedListening();
      _addSelectImageListening();
    } else {
      final element =
          FReportInfo.instance.getElementInfoById(inputImageList!.id ?? '');
      if (element != null) {
        _inputImageListInfo = element as InputImageListInfo;
        _addSelectImageListening();
      }
    }
  }

  void _checkInputImageListInfo() {
    if (_inputImageListInfo != null) {
      final imageInfo =
          FReportInfo.instance.getBlockElement(widget.inputImageList);
      if (imageInfo != null) {
        imageInfo as InputImageListInfo;
      } else {
        final element =
            FReportInfo.instance.getElementInfoById(inputImageList!.id ?? '');
        if (element != null) {
          _inputImageListInfo = element as InputImageListInfo;
          _addSelectImageListening();
        }
      }
      if (imageInfo != _inputImageListInfo && imageInfo is InputImageListInfo) {
        if (_inputImageListInfo != null) {
          _inputImageListInfo!.onSelectedChange.dispose();
        }
        _inputImageListInfo = imageInfo;
        _addSelectImageListening();

        _addOnSelectedListening();
        if (_images != _inputImageListInfo!.selectedImages) {
          _images = _inputImageListInfo!.selectedImages;
        }
        _isSelected = false;
        _child = _getImageListChild();
      } else {
        _addOnSelectedListening();
        if (_images != _inputImageListInfo!.selectedImages) {
          setState(() {
            _images = _inputImageListInfo!.selectedImages;
            _child = _getImageListChild();
          });
          _heightAdaptation();
        }
      }
    }
  }

  void _onSelectedInputChange() {
    if (_isSelected) {
      setState(() {
        _borderColor = Colors.grey;
        _isSelected = false;
        _inputImageListInfo!.isSelected = false;
      });
    } else {
      setState(() {
        _borderColor = const Color.fromARGB(255, 64, 159, 248);
        _isSelected = true;
        _inputImageListInfo!.isSelected = true;
      });
    }
    _heightAdaptation();
  }

  void _heightAdaptation() {
    var cellPostion = widget.cellPostion;
    if (cellPostion != null) {
      final rowCount = (_images.length / inputImageList!.column!).ceil();
      //触发高度适配
      if (widget.parentTableId.isNotEmpty) {
        FReportInfo.instance.cellRowChange.emit(
          this,
          TableScaleInfo(
            rowCount == 0 ? 1 : rowCount.toDouble(),
            cellPostion,
            widget.parentTableId,
          ),
        );
      }
    }
  }
}
