import 'package:fis_lib_report/converts/pt_to_px_converter.dart';
import 'package:fis_lib_report/converts/text_size_converter.dart';
import 'package:fis_lib_report/intents/auto_jump_intent.dart';
import 'package:fis_lib_report/report/cell_postion.dart';
import 'package:fis_lib_report/report/element_tag.dart';
import 'package:fis_lib_report/report/input_text.dart';
import 'package:fis_lib_report/report_info/cell_position_info.dart';
import 'package:fis_lib_report/report_info/element_tag_names.dart';
import 'package:fis_lib_report/report_info/input_text_info.dart';
import 'package:fis_lib_report/report_info/report_info.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:fis_ui/values/spacings.dart';
import 'package:fis_ui/widgets/gesture/actions.dart';
import 'package:fis_ui/widgets/gesture/shortcuts.dart';
import 'package:fis_ui/widgets/layout/button_group.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class FInputText extends FStatefulWidget implements FInteractiveContainer {
  final InputText inputText;
  @override
  final String pageName = 'FInputText';
  final CellPostion? cellPostion;
  final String parentTableId;
  const FInputText({
    Key? key,
    required this.inputText,
    this.cellPostion,
    this.parentTableId = "",
  }) : super(key: key);

  @override
  FState<FStatefulWidget> createState() {
    return _FInputTextState();
  }
}

class _FInputTextState extends FState<FInputText> {
  _FInputTextState();
  final _controller = CustomTextEditingController();
  final FocusNode _focusNode = FocusNode();
  final ButtonStyle _buttonStyle = const ButtonStyle(
    padding: MaterialStatePropertyAll(EdgeInsets.all(0)),
  );
  double _lineWidth = 136.0;
  double _fontSize = 22.0;
  double _allHeight = 0.0;
  bool _textWrap = false;
  Color _fontColor = const Color.fromARGB(255, 0, 0, 0);
  Color _backgroundColor = const Color.fromARGB(255, 255, 255, 255);
  TextStyle? _textStyle;
  InputTextInfo? _inputTextInfo;
  bool _isReadOnly = false;
  bool _widthLock = false;
  int _rowCount = 1;
  InputDecoration _inputDecoration =
      const InputDecoration(border: OutlineInputBorder());
  double? _freezeHeight; // 输入框高度冻结
  /// 组件Key
  final GlobalKey _textFieldConatinerKey = GlobalKey();

  @override
  initState() {
    _focusNode.addListener(_textFieldFocusNodeListener);
    _controller.addComposeListener(_composeListener);
    _initStyle();
    WidgetsBinding.instance.addPostFrameCallback((_) {
      _heightAdaptation();
    });
    super.initState();
  }

  /// 焦点事件监听
  void _textFieldFocusNodeListener() {
    if (!_focusNode.hasFocus) {
      _onInputChanged(_textStyle!, _controller.text);
    }
  }

  // 开始进行中文输入时触发
  void _composeListener(bool isComposing) {
    if (isComposing) {
      final RenderBox renderBox = _textFieldConatinerKey.currentContext!
          .findRenderObject() as RenderBox;
      setState(() {
        _freezeHeight = renderBox.size.height;
      });
    } else {
      setState(() {
        _freezeHeight = null;
      });
    }
  }

  @override
  void didUpdateWidget(FInputText oldWidget) {
    _initStyle();
    WidgetsBinding.instance.addPostFrameCallback((_) {
      _heightAdaptation();
    });
    super.didUpdateWidget(oldWidget);
  }

  @override
  FWidget build(BuildContext context) {
    _checkInputTextInfo();
    final textField = FShortcuts(
      shortcuts: {
        LogicalKeySet(LogicalKeyboardKey.tab): AutoJumpIntent(),
      },
      child: FActions(
        actions: <Type, Action<Intent>>{
          AutoJumpIntent: CallbackAction<AutoJumpIntent>(
            onInvoke: (AutoJumpIntent intent) {
              intent.handleAutoJump(_controller);
              return null;
            },
          ),
        },
        child: FTextField(
          textAlignVertical: TextAlignVertical.top,
          decoration: _isReadOnly ? null : _inputDecoration,
          readOnly: _isReadOnly,
          focusNode: _focusNode,
          minLines: _textWrap ? _rowCount : 1,
          maxLines:
              (_textWrap && widget.cellPostion == null) ? null : _rowCount,
          controller: _controller,
          onEditingComplete: () {
            _onInputChanged(_textStyle!, _controller.text);
          },
          onChanged: (value) {
            _inputTextInfo!.writeOnly = value;
          },
          style: _textStyle,
        ),
      ),
    );
    final suffixIcon = _buildSufficIcon();
    FWidget body;
    if (suffixIcon != null) {
      body = FStack(
        children: [
          textField,
          FPositioned(
            right: 6,
            bottom: 6,
            child: suffixIcon,
          ),
        ],
      );
    } else {
      body = textField;
    }
    return FColumn(
      mainAxisSize: MainAxisSize.min,
      children: [
        FContainer(
          key: _textFieldConatinerKey,
          width: _lineWidth,
          height: _textWrap ? _freezeHeight : _allHeight,
          alignment: Alignment.center,
          child: body,
        ),
        if (_textWrap) ...[
          const FSizedBox(height: 3),
        ],
        if (_inputTextInfo != null &&
            _inputTextInfo!.tag != null &&
            _inputTextInfo!.tag!.elementTagType ==
                ElementTagType.DiagnosticEntry) ...[
          FAlign(
            alignment: Alignment.centerRight,
            child: FContainer(
              margin: const EdgeInsets.only(right: 5),
              child: FButtonGroup(
                spacing: FSpacings.MINI_BUTTON,
                children: [
                  FElevatedButton(
                      businessParent: widget,
                      name: "diagnosticTap",
                      onPressed: () {
                        FReportInfo.instance.onDiagnosticTap
                            .emit(this, _inputTextInfo!.tag!.name ?? '');
                      },
                      child: FText(FReportInfo.instance.selectEntry),
                      style: _buttonStyle),
                  FElevatedButton(
                    businessParent: widget,
                    name: "revoke",
                    onPressed: () {
                      setState(() {
                        _inputTextInfo!.text = '';
                      });
                    },
                    child: FText(FReportInfo.instance.revoke),
                    style: _buttonStyle,
                  ),
                ],
              ),
            ),
          ),
        ],
      ],
    );
  }

  //onchange 事件
  void _onInputChanged(TextStyle _textStyle, String value) {
    if (_inputTextInfo != null) {
      _inputTextInfo!.text = value;
    }
  }

  @override
  void dispose() {
    _inputTextInfo?.onTextChange.dispose();
    _controller.dispose();
    _focusNode.removeListener(_textFieldFocusNodeListener);
    _controller.removeComposeListener(_composeListener);
    super.dispose();
  }

  void _initDatas() {
    final inputText = widget.inputText;
    _controller.text = '';

    final inputTextInfo = FReportInfo.instance.getElementInfo(inputText);
    _inputTextInfo = inputTextInfo as InputTextInfo;
    if (_inputTextInfo!.isReadOnly!) {
      if (_inputTextInfo!.tag!.name == TagNames.RPHYSICIAN) {
        _controller.text = FReportInfo.instance.reporter ?? '';
      }
    }

    final fontColor = inputText.fontColor;
    if (fontColor != null) {
      _fontColor = Color.fromARGB(
          fontColor.a!, fontColor.r!, fontColor.g!, fontColor.b!);
    }
    final backgroundColor = inputText.background;
    if (backgroundColor != null) {
      _backgroundColor = Color.fromARGB(backgroundColor.a!, backgroundColor.r!,
          backgroundColor.g!, backgroundColor.b!);
    }
    _inputDecoration = InputDecoration(
      contentPadding: EdgeInsets.all(_textWrap ? 8 : 4),
      border: const OutlineInputBorder(),
    );
    FReportInfo.instance.onReloadFinsh.addListener((sender, e) {
      _initDatas();
    });
    setState(() {});
  }

  void _syncWidth(InputText inputText, String value) {
    final fontSize = _textStyle!.fontSize ?? 18;
    final width = fontSize * value.length + 15;

    if (inputText.tag != null && inputText.tag!.name == TagNames.HOSPITALNAME) {
      if (_lineWidth < width) {
        setState(() {
          if (width < 480) {
            _lineWidth = width;
            _widthLock = true;
          }
        });
      } else if (_controller.text.isEmpty) {
        setState(() {
          //重置长度
          _lineWidth = PtToPxConverter.ptToPx(inputText.lineWidth);
          _widthLock = false;
        });
      } else if (TextSizeConvert.getTextSize(_controller.text, _textStyle!)
              .width >
          inputText.lineWidth!) {
        setState(() {
          if (width < 480) {
            _lineWidth = width;
            _widthLock = true;
          }
        });
      }
    }
  }

  void _initStyle() {
    final inputText = widget.inputText;
    _textStyle ??= TextStyle(
      fontSize: PtToPxConverter.ptToPx(inputText.fontSize),
      color: _fontColor,
    );
    _isReadOnly = inputText.isReadOnly;
    //TODO(Loki):set FontName in TextField
    final fontName = inputText.fontName;
    //TODO(Loki):常规模板暂未设置fontStyles，后续再支持
    final fontStyles = inputText.fontStyles;
    if (!_widthLock) {
      _lineWidth = PtToPxConverter.ptToPx(inputText.lineWidth);
    }
    _widthLock = false;
    _textWrap = inputText.textWrap ?? false;
    _rowCount = _textWrap ? 6 : 1;
    if (_inputTextInfo != null && _inputTextInfo!.text.isNotEmpty) {
      final text = _inputTextInfo!.text;
      _syncWidth(
        widget.inputText,
        text,
      );
    }

    _fontSize = PtToPxConverter.ptToPx(inputText.fontSize);
    _allHeight = _textWrap ? (10 + _fontSize) * _rowCount : _fontSize;
    _allHeight += 10;
    if (_inputTextInfo != null && _textWrap) {
      _allHeight += 16;
    }

    _inputDecoration = InputDecoration(
      contentPadding: EdgeInsets.all(_textWrap ? 8 : 4),
      border: const OutlineInputBorder(),
    );
  }

  void _checkInputTextInfo() {
    try {
      final inputTextInfo =
          FReportInfo.instance.getElementInfo(widget.inputText);
      if (_inputTextInfo != inputTextInfo && inputTextInfo != null) {
        if (_inputTextInfo != null) _inputTextInfo!.onTextChange.dispose();
        _inputTextInfo = inputTextInfo as InputTextInfo;
        _addListining();
      } else if (inputTextInfo != null) {
        _inputTextInfo!.onTextChange.dispose();
        _addListining();
      }
      if (!_inputTextInfo!.isListening!) {
        _initDatas();
        _inputTextInfo!.isListening = true;
      }
      if (_inputTextInfo!.text != _controller.text) {
        _controller.text = _inputTextInfo!.text;
      }
    } catch (e) {
      print(e);
    }
  }

  void _addListining() {
    _inputTextInfo!.onTextChange.addListener((sender, e) {
      setState(() {
        _controller.text = e;
        _syncWidth(
          widget.inputText,
          e,
        );
      });
    });
  }

  FWidget? _buildSufficIcon() {
    if (_inputTextInfo == null || _inputTextInfo!.featureTag.isEmpty) {
      return null;
    }
    if (_inputTextInfo!.featureTag.contains(TagNames.SPEECHINPUT) &&
        FReportInfo.instance.isASROn) {
      return FMouseRegion(
        cursor: SystemMouseCursors.click,
        child: FGestureDetector(
          child: const FIcon(Icons.mic, size: 20),
          onTap: () {
            FReportInfo.instance.onSpeechInput.emit(this, _inputTextInfo!);
          },
          businessParent: widget,
          name: TagNames.SPEECHINPUT,
        ),
      );
    }
    return null;
  }

  void _heightAdaptation() {
    var cellPostion = widget.cellPostion;
    if (cellPostion != null) {
      final rowHeightScale = _rowCount > 5 ? 1.5 : 1.0;
      //触发高度适配
      if (widget.parentTableId.isNotEmpty) {
        FReportInfo.instance.cellRowChange.emit(
          this,
          TableScaleInfo(
            rowHeightScale < 1 ? 1 : rowHeightScale,
            cellPostion,
            widget.parentTableId,
          ),
        );
      }
    }
  }
}

/// 自定义TextEditingController，会在输入法开始输入时触发监听
class CustomTextEditingController extends TextEditingController {
  final List<ValueChanged<bool>> _composeListeners = [];

  void addComposeListener(ValueChanged<bool> listener) {
    _composeListeners.add(listener);
  }

  void removeComposeListener(ValueChanged<bool> listener) {
    _composeListeners.remove(listener);
  }

  @override
  set value(TextEditingValue newValue) {
    final bool isComposingBefore = value.composing.isValid;
    final bool isComposingAfter = newValue.composing.isValid;

    if (isComposingBefore != isComposingAfter) {
      _notifyComposeListeners(isComposingAfter);
    }

    super.value = newValue;
  }

  void _notifyComposeListeners(bool isComposing) {
    for (final listener in _composeListeners) {
      listener(isComposing);
    }
  }
}
