import 'package:fis_lib_report/converts/pt_to_px_converter.dart';
import 'package:fis_lib_report/report/single_selected.dart';
import 'package:fis_lib_report/report_info/report_info.dart';
import 'package:fis_lib_report/report_info/single_selected_info.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_ui/widgets/custom_dropdown_button.dart' as custom_widget;
import 'package:flutter/material.dart';
import 'package:fis_theme/theme.dart';
// import 'package:dropdown_button2/dropdown_button2.dart';

class FSingleSelected extends StatefulWidget implements FWidget {
  final SingleSelected singleSelected;
  const FSingleSelected(this.singleSelected, {Key? key}) : super(key: key);
  @override
  State<StatefulWidget> createState() {
    return _RSingleSelectState();
  }
}

class _RSingleSelectState extends State<FSingleSelected> {
  late SingleSelected singleSelected;
  _RSingleSelectState();
  String _value = '';
  List<String>? _items = [];
  SingleSelectedInfo? _singleSelectedInfo;
  late final double _width;
  double _height = 24;
  double _fontSize = 0;

  @override
  initState() {
    _width = PtToPxConverter.ptToPx(widget.singleSelected.lineWidth);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    singleSelected = widget.singleSelected;
    final selectInfo = FReportInfo.instance.getElementInfo(singleSelected);
    if (selectInfo != null && _singleSelectedInfo != selectInfo) {
      _singleSelectedInfo = selectInfo as SingleSelectedInfo;
      _singleSelectedInfo!.onSelectedChange.addListener((sender, e) {
        if (_singleSelectedInfo!.items!.contains(e)) {
          if (mounted) {
            setState(() {
              _value = _singleSelectedInfo!.selectedItem;
            });
          }
        }
      });
    }
    if (singleSelected.items!.isNotEmpty) {
      _items = singleSelected.items;
    }
    if (_items!.isEmpty) {
      return const FSizedBox();
    }
    final background = singleSelected.background!;
    if (_singleSelectedInfo != null &&
        _singleSelectedInfo!.selectedItem.isNotEmpty &&
        _singleSelectedInfo!.selectedItem != _value) {
      _value = _singleSelectedInfo!.selectedItem;
    }
    _fontSize = PtToPxConverter.ptToPx(singleSelected.fontSize);
    _height = _fontSize + 10;
    return SizedBox(
      height: _height,
      child: ButtonTheme(
        /// 为了让下拉框的宽度和输入框一样
        alignedDropdown: true,
        child: DropdownButtonHideUnderline(
          child: custom_widget.DropdownButtonFormField<String>(
            itemHeight: _height,
            iconSize: 18,
            isExpanded: false,
            decoration: InputDecoration(
                contentPadding:
                    const EdgeInsets.symmetric(horizontal: 0, vertical: 2),
                constraints: BoxConstraints(
                  maxWidth: _width,
                ),
                border: const OutlineInputBorder(
                  borderSide: BorderSide(color: Colors.grey, width: 0.8),
                  borderRadius: BorderRadius.all(Radius.circular(4.0)),
                ),
                focusedBorder: const OutlineInputBorder(
                  borderSide: BorderSide(color: Colors.grey, width: 0.8),
                  borderRadius: BorderRadius.all(Radius.circular(4.0)),
                ),
                fillColor: Color.fromARGB(background.a!, background.r!,
                    background.g!, background.b!)),
            selectedItemBuilder: (_) => _items!
                .map((e) => _OptionRow(
                      singleSelected,
                      e,
                      height: _height,
                      maxWidth: _width,
                    ))
                .toList(),
            value: _value.isEmpty ? null : _value,
            items: buildItems(),
            onChanged: (v) {
              setState(() {
                if (_singleSelectedInfo != null) {
                  _singleSelectedInfo!.selectedItem = v!;
                }
                _value = v!;
              });
            },
          ),
        ),
      ),
    );
  }

  List<custom_widget.DropdownMenuItem<String>>? buildItems() {
    return _items!.map((String e) {
      final active = e == _value;
      return custom_widget.DropdownMenuItem<String>(
        child: _OptionRow(
          singleSelected,
          e,
          isActive: active,
          height: _height,
        ),
        value: e,
      );
    }).toList();
  }
}

class _OptionRow extends StatelessWidget implements FWidget {
  const _OptionRow(
    this.singleSelected,
    this.label, {
    Key? key,
    this.isActive = false,
    this.height,
    this.maxWidth,
  }) : super(key: key);

  final SingleSelected singleSelected;
  final String label;
  final bool isActive;
  final double? height;
  final double? maxWidth;
  @override
  FWidget build(BuildContext context) {
    final color = singleSelected.fontColor!;

    final text = FText(
      label,
      overflow: TextOverflow.ellipsis,
      style: TextStyle(
        fontSize: PtToPxConverter.ptToPx(singleSelected.fontSize),
        color: Color.fromARGB(color.a!, color.r!, color.g!, color.b!),
      ),
    );
    var defaultPadding = const EdgeInsets.only(left: 4);
    if (height != null) {
      return FContainer(
        alignment: Alignment.centerLeft,
        padding: defaultPadding,
        height: height,
        constraints: BoxConstraints(
          maxWidth: maxWidth == null ? double.infinity : maxWidth! - 26,
        ),
        child: text,
        color: isActive ? FTheme.ins.colorScheme.secondary : null,
      );
    } else {
      return FContainer(
        padding: defaultPadding,
        child: text,
      );
    }
  }
}
