import 'package:fis_lib_report/converts/alignment_convert.dart';
import 'package:fis_lib_report/pages/components/datetime.dart';
import 'package:fis_lib_report/pages/components/input_text.dart';
import 'package:fis_lib_report/pages/components/line.dart';
import 'package:fis_lib_report/pages/components/multi_selected.dart';
import 'package:fis_lib_report/pages/components/multi_selected_vertical.dart';
import 'package:fis_lib_report/pages/components/page_number.dart';
import 'package:fis_lib_report/pages/components/single_select.dart';
import 'package:fis_lib_report/pages/components/space.dart';
import 'package:fis_lib_report/pages/components/static_image.dart';
import 'package:fis_lib_report/pages/components/static_text.dart';
import 'package:fis_lib_report/pages/helper.dart';
import 'package:fis_lib_report/report/cell_postion.dart';
import 'package:fis_lib_report/report/date_time_element.dart';
import 'package:fis_lib_report/report/element_type.dart';
import 'package:fis_lib_report/report/input_image.dart';
import 'package:fis_lib_report/report/input_text.dart';
import 'package:fis_lib_report/report/interfaces/element.dart';
import 'package:fis_lib_report/report/line.dart';
import 'package:fis_lib_report/report/multi_selected.dart';
import 'package:fis_lib_report/report/multi_selected_vertical.dart';
import 'package:fis_lib_report/report/page_number.dart';
import 'package:fis_lib_report/report/paragraph.dart';
import 'package:fis_lib_report/report/rt_Cell.dart';
import 'package:fis_lib_report/report/rt_table.dart';
import 'package:fis_lib_report/report/single_selected.dart';
import 'package:fis_lib_report/report/space.dart';
import 'package:fis_lib_report/report/static_text.dart';
import 'package:fis_lib_report/report/static_image.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:fis_i18n/i18n.dart';
import 'components/input_image.dart';

class FParagraphPage extends StatefulWidget implements FWidget {
  final Paragraph paragraph;
  final CellPostion? cellPostion;
  final String parentTableId;
  const FParagraphPage(
      {Key? key,
      required this.paragraph,
      this.cellPostion,
      this.parentTableId = ""})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _ParagraphState();
  }
}

class _ParagraphState extends State<FParagraphPage> {
  List<IElement> _elements = [];
  int _itemCount = 0;

  @override
  initState() {
    super.initState();
  }

  @override
  FWidget build(BuildContext context) {
    _elements = widget.paragraph.elements ?? [];
    _itemCount = _elements.length;
    if (_itemCount == 0) {
      return FContainer(
        constraints: const BoxConstraints(minHeight: 0),
      );
    } else if (_itemCount == 2 &&
        _elements[0].elementType!.name == ElementType.staticText.name &&
        (_elements[1].elementType!.name == ElementType.singleSelected.name ||
            _elements[1].elementType!.name == ElementType.multiSelected.name ||
            _elements[1].elementType!.name == ElementType.dateTime.name ||
            _elements[1].elementType!.name == ElementType.inputText.name)) {
      try {
        final staticText = _elements[0];
        if (staticText.parent != null) {
          final paragraph = staticText.parent as Paragraph;
          if (paragraph.parent != null) {
            final cell = paragraph.parent as RTCell;
            if (cell.parent != null) {
              final table = cell.parent as RTTable;
              final values = table.cells!.values.toList();
              for (var i = 0; i < values.length; i++) {
                if (values[i] == cell) {
                  final key = table.cells!.keys.toList()[i];
                  final column = key.column;
                  final width = table.columnDefinitions![column!].width;
                  var valueWidth = 0.0;
                  if (_elements[1].elementType!.name ==
                      ElementType.inputText.name) {
                    final input = _elements[1] as InputText;
                    valueWidth = input.lineWidth ?? 0;
                  }
                  if (_elements[1].elementType!.name ==
                      ElementType.singleSelected.name) {
                    final input = _elements[1] as SingleSelected;
                    valueWidth = input.lineWidth ?? 0;
                    input.fontSize = 9;
                    if ((input.items?.any((element) => element.length > 7) ??
                            false) &&
                        i18nBook.isCurrentChinese) {
                      valueWidth += 13;
                      input.lineWidth = input.lineWidth! + 13;
                    }
                  }
                  if (_elements[1].elementType!.name ==
                      ElementType.multiSelected.name) {
                    final input = _elements[1] as MultiSelected;
                    valueWidth = input.lineWidth ?? 0;
                  }
                  if (_elements[1].elementType!.name ==
                      ElementType.dateTime.name) {
                    final dateTime = _elements[1] as DateTimeElement;
                    if (!dateTime.isReadOnly) {
                      valueWidth = dateTime.lineWidth ?? 0;
                    }
                  }
                  if (valueWidth > 0) {
                    final textElement = staticText as StaticText;
                    final textWidth = width! - valueWidth;
                    textElement.lineWidth = textWidth;
                  }
                }
              }
            }
          }
        }
      } catch (e) {
        print(e);
      }
    }
    final margin = widget.paragraph.margin!;
    return FContainer(
      margin: EdgeInsets.only(
        top: margin.top!,
        bottom: margin.bottom!,
        left: margin.left!,
        right: margin.right!,
      ),
      alignment: AlignmentConvert.horizontalConvert(
          widget.paragraph.horizontalAlignment),
      child: FWrap(
        alignment: WrapAlignment.start,
        direction: Axis.horizontal,
        crossAlignment: WrapCrossAlignment.center,
        children: [
          ..._elements.map((element) => _buildElementWidget(element)),
        ],
      ),
    );
  }

  FWidget _buildElementWidget(IElement element) {
    FWidget? eleWidget;
    final typeName = element.elementType!.name;
    if (typeName == ElementType.inputText.name) {
      InputText inputText = element as InputText;
      eleWidget = FInputText(
        inputText: inputText,
        cellPostion: widget.cellPostion,
        parentTableId: widget.parentTableId,
      );
    } else if (typeName == ElementType.staticText.name) {
      StaticText staticText = element as StaticText;
      eleWidget = FStaticText(staticText);
    } else if (typeName == ElementType.singleSelected.name) {
      SingleSelected singleSelected = element as SingleSelected;
      eleWidget = FSingleSelected(singleSelected);
    } else if (typeName == ElementType.line.name) {
      Line line = element as Line;
      eleWidget = FLine(line);
    } else if (typeName == ElementType.dateTime.name) {
      final dateTime = element as DateTimeElement;
      eleWidget = FDateTime(dateTime);
    } else if (typeName == ElementType.multiSelected.name) {
      final multiSelected = element as MultiSelected;
      eleWidget = FMultiSelected(multiSelected);
    } else if (typeName == ElementType.multiSelectedVertical.name) {
      final multiSelected = element as MultiSelectedVertical;
      eleWidget = FMultiSelectedVertical(multiSelected);
    } else if (typeName == ElementType.inputImage.name) {
      final inputImage = element as InputImage;
      eleWidget = FInputImage(inputImage);
    } else if (typeName == ElementType.staticImage.name) {
      final staticImage = element as StaticImage;
      eleWidget = FStaticImage(staticImage);
    } else if (typeName == ElementType.space.name) {
      final space = element as Space;
      eleWidget = FSpace(space);
    } else if (typeName == ElementType.pageNumber.name) {
      final pageNumber = element as PageNumber;
      eleWidget = FPageNumber(pageNumber);
    }
    if (eleWidget == null) {
      return FContainer(
        height: 30,
        width: 180,
        decoration: TestBoxDecoration.buildDecoration(),
        child: const FText('Unknown component placeholder'),
      );
    }
    return eleWidget;
  }
}
