import 'package:fis_lib_report/pages/paragraph_page.dart';
import 'package:fis_lib_report/pages/rt_table.dart';
import 'package:fis_lib_report/report/cell_postion.dart';
import 'package:fis_lib_report/report/element_type.dart';
import 'package:fis_lib_report/report/input_image_list.dart';
import 'package:fis_lib_report/report/interfaces/block_element.dart';
import 'package:fis_lib_report/report/paragraph.dart';
import 'package:fis_lib_report/report/rt_Cell.dart';
import 'package:fis_lib_report/report/rt_table.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'components/input_imageList.dart';
import 'helper.dart';

class FRTCellPage extends StatefulWidget implements FWidget {
  final double? width;
  final RTCell cell;
  final CellPostion? cellPostion;
  final String parentTableId;

  const FRTCellPage(
      {required this.cell,
      this.width,
      this.cellPostion,
      this.parentTableId = "",
      Key? key})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _RTCellState();
  }
}

class _RTCellState extends State<FRTCellPage> {
  List<IBlockElement> get _blocks => widget.cell.blocks ?? [];

  @override
  initState() {
    super.initState();
  }

  @override
  FWidget build(BuildContext context) {
    return _blocks.isEmpty
        ? FContainer(
            constraints: const BoxConstraints(minHeight: 24),
            width: widget.width,
          )
        : FWrap(
            alignment: WrapAlignment.center,
            runAlignment: WrapAlignment.center,
            runSpacing: 2,
            children: [
              ..._blocks.map((e) {
                if (e.elementType!.name == ElementType.paragraph.name) {
                  final paragraph = e as Paragraph;
                  return FContainer(
                    width: widget.width,
                    alignment: Alignment.center,
                    child: FParagraphPage(
                      paragraph: paragraph,
                      cellPostion: widget.cellPostion,
                      parentTableId: widget.parentTableId,
                    ),
                  );
                } else if (e.elementType!.name == ElementType.rtTable.name) {
                  final table = e as RTTable;
                  return FSizedBox(
                    width: widget.width,
                    child: FRTTablePage(element: table),
                  );
                } else if (e.elementType!.name == ElementType.imageList.name) {
                  final inputImageList = e as InputImageList;
                  return FCenter(
                    child: FInputImageList(inputImageList, widget.cellPostion,
                        parentTableId: widget.parentTableId),
                  );
                }
                return FContainer();
              }),
            ],
          );
  }
}
