import 'dart:math';

import 'package:fis_lib_report/converts/alignment_convert.dart';
import 'package:fis_lib_report/converts/color_convert.dart';
import 'package:fis_lib_report/converts/pt_to_px_converter.dart';
import 'package:fis_lib_report/pages/components/static_text.dart';
import 'package:fis_lib_report/pages/rt_cell.dart';
import 'package:fis_lib_report/report/cell_postion.dart';
import 'package:fis_lib_report/report/element_type.dart';
import 'package:fis_lib_report/report/interfaces/cell.dart';
import 'package:fis_lib_report/report/interfaces/grid.dart';
import 'package:fis_lib_report/report/paragraph.dart';
import 'package:fis_lib_report/report/rt_Cell.dart';
import 'package:fis_lib_report/report/rt_border.dart';
import 'package:fis_lib_report/report/rt_table.dart';
import 'package:fis_lib_report/report/static_image.dart';
import 'package:fis_lib_report/report_info/cell_position_info.dart';
import 'package:fis_lib_report/report_info/report_info.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';

class FRTTablePage extends StatefulWidget implements FWidget {
  const FRTTablePage({required this.element, Key? key}) : super(key: key);

  final RTTable element;

  @override
  State<StatefulWidget> createState() {
    return _RTTableState();
  }
}

class _RTTableState extends State<FRTTablePage> {
  Map<CellPostion, ICell> get _cells => widget.element.cells ?? {};
  int? _columnCount;
  int? _rowCount;
  double _width = 0;
  double _height = 0;
  double _rowHeightScale = 1;
  String get _tableId => widget.element.id ?? '';

  /// 如果置空容易出现误收通知
  final List<RTRowDefinition> _rowDefinitions = [];

  @override
  initState() {
    _rowDefinitions.clear();
    for (RTRowDefinition row in widget.element.rowDefinitions ?? []) {
      _rowDefinitions.add(RTRowDefinition(
        row.height,
        row.type,
        row.minHeight,
      ));
    }
    initCells();
    super.initState();
    FReportInfo.instance.cellRowChange.addListener(_onCellRowChange);
  }

  @override
  void didUpdateWidget(FRTTablePage oldWidget) {
    initCells();
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    FReportInfo.instance.cellRowChange.removeListener(_onCellRowChange);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    ///这个Map用于表格结构
    Map<CellPostion, FWidget> gridList = {};

    ///这个children 用于普通RTTable
    List<FWidget> children = [];
    for (int i = 0; i < _rowCount!; i++) {
      final rowChildren =
          _cells.keys.where((element) => element.row! == i).toList();

      // FWidget row = _buildRowWidget(i);
      // rowWidgets.add(row);

      var rowItems = rowChildren.toList();

      ///排序：按照所在列从0开始排
      rowItems.sort(((a, b) => (a.column ?? 0).compareTo(b.column ?? 0)));
      List<FWidget> rowNormalChildren = [];
      for (var cellPostion in rowItems) {
        ///一行元素集合中的一个元素
        FWidget childrenItem = _buildChildrenItem(cellPostion, rowItems, i);
        gridList[cellPostion] = childrenItem;
        rowNormalChildren.add(FContainer(
          child: childrenItem,
        ));
      }
      children.add(
        FRow(
          children: rowNormalChildren,
        ),
      );
    }

    var columnCount = _columnCount ?? 3;
    if (_width <= 0) {
      initCells();
    }

    const int expansionFactor = 1000;
    Widget table = StaggeredGrid.count(
      ///水平方向
      crossAxisCount: columnCount * expansionFactor,
      mainAxisSpacing: 0,
      children: gridList.keys.map((e) {
        var currentItemWidth =
            widget.element.columnDefinitions![e.column!].width ?? 0;
        if (e.columnSpan! > 1) {
          for (int i = e.column! + 1; i < e.column! + e.columnSpan!; i++) {
            currentItemWidth += widget.element.columnDefinitions![i].width ?? 0;
          }
        }
        var currentItemHeight = _rowDefinitions[e.row!].height ?? 0;

        if (e.rowSpan! > 1 && e.row! + 1 < _rowDefinitions.length) {
          for (int i = e.row! + 1; i < e.row! + e.rowSpan!; i++) {
            var targetHeight = _rowDefinitions[i].height ?? 0;
            currentItemHeight += targetHeight;
          }
        }
        double currentColumnRatio = 0;
        double currentRowRatio = 0;
        if (_width > 0) {
          currentColumnRatio =
              currentItemWidth / _width * expansionFactor * columnCount;
        } else {
          currentColumnRatio = e.columnSpan! * expansionFactor.toDouble();
        }

        if (_height > 0) {
          currentRowRatio =
              currentItemHeight / _width * expansionFactor * columnCount;
        } else {
          currentRowRatio = e.rowSpan!.toDouble() * expansionFactor;
        }
        return StaggeredGridTile.count(
          ///水平方向
          crossAxisCellCount: currentColumnRatio.toInt(),

          ///竖直方向
          mainAxisCellCount: currentRowRatio,
          child: Container(
            child: gridList[e]!,
          ),
        );
      }).toList(),
    );

    return Container(
      alignment: AlignmentConvert.horizontalConvert(
          widget.element.horizontalAlignment),
      child: table,
    );
  }

  void initCells() {
    _width = 0;
    _height = 0;
    _rowHeightScale = 1;
    _rowDefinitions.clear();
    for (RTRowDefinition row in widget.element.rowDefinitions ?? []) {
      _rowDefinitions.add(RTRowDefinition(
        row.height,
        row.type,
        row.minHeight,
      ));
    }
    List<CellPostion> cellPostions = _cells.keys.toList();
    cellPostions.sort(((a, b) => (b.column!).compareTo(a.column!)));
    // +1 为列数
    _columnCount = cellPostions.first.column! + 1;
    for (int i = 0; i < _columnCount!; i++) {
      _width += widget.element.columnDefinitions![i].width!;
    }
    cellPostions.sort(((a, b) => (b.row!).compareTo(a.row!)));
    // +1 为行数
    _rowCount = cellPostions.first.row! + 1;
    for (int i = 0; i < _rowCount!; i++) {
      _height += _rowDefinitions[i].height!;
    }
    setState(() {});
  }

  FWidget _buildChildrenItem(
    CellPostion cellPostion,
    List<CellPostion> rowItems,
    int index,
  ) {
    double width =
        widget.element.columnDefinitions![cellPostion.column!].width ?? 0.0;
    double height = _rowDefinitions[cellPostion.row!].height ?? 0.0;

    ///columnSpan是横向合并列数的
    if (cellPostion.columnSpan! > 1) {
      for (int i = 1; i < cellPostion.columnSpan!; i++) {
        width +=
            widget.element.columnDefinitions![cellPostion.column! + i].width ??
                0.0;
      }
    }

    ///rowSpan是竖向合并的行数
    if (cellPostion.rowSpan! > 1) {
      for (int i = 1; i < cellPostion.rowSpan!; i++) {
        height += _rowDefinitions[cellPostion.row! + i - 1].height ?? 0.0;
      }
    }

    final cell = _cells[cellPostion];
    RTCell rtCell = cell as RTCell;
    final borders = rtCell.borders!;
    final background = rtCell.background;
    var childBlock = rtCell.blocks ?? [];

    if (childBlock.isNotEmpty) {
      for (var element in childBlock) {
        if (element.elementType!.name == ElementType.paragraph.name) {
          var para = element as Paragraph;
          var paraEle = para.elements ?? [];
          if (paraEle.isNotEmpty) {
            for (var ele in paraEle) {
              ///这里是修正静态图像的尺寸的
              if (ele is StaticImage) {
                if (ele.imageWidth > width) {
                  width = ele.imageWidth;
                }
                if (height < ele.imageHeight) {
                  height = ele.imageHeight;
                }
              }
            }
          }
        }
      }
    }

    var realWidth = PtToPxConverter.ptToPx(width);
    FWidget cellItem = FContainer(
      decoration: BoxDecoration(
        color: ColorConvert.colorConvert(background),
        border: Border(
          left: BorderSide(
              style: borders.left.borderStyle == RTBorderStyle.None
                  ? BorderStyle.none
                  : BorderStyle.solid,
              width: borders.left.thickness,
              color: ColorConvert.colorConvert(borders.left.color)),
          right: BorderSide(
              style: borders.right.borderStyle == RTBorderStyle.None
                  ? BorderStyle.none
                  : BorderStyle.solid,
              width: borders.right.thickness,
              color: ColorConvert.colorConvert(borders.right.color)),
          top: BorderSide(
              style: borders.top.borderStyle == RTBorderStyle.None
                  ? BorderStyle.none
                  : BorderStyle.solid,
              width: borders.top.thickness,
              color: ColorConvert.colorConvert(borders.top.color)),
          bottom: BorderSide(
              style: borders.bottom.borderStyle == RTBorderStyle.None
                  ? BorderStyle.none
                  : BorderStyle.solid,
              width: borders.bottom.thickness,
              color: ColorConvert.colorConvert(borders.bottom.color)),
        ),
      ),
      //height: realHeight,
      width: realWidth,
      child: FRTCellPage(
        cell: rtCell,
        width: realWidth,
        cellPostion: cellPostion,
        parentTableId: _tableId,
      ),
    );

    return cellItem;
  }

  void _onCellRowChange(Object sender, TableScaleInfo e) {
    if (_tableId == e.receiverId) {
      try {
        var currentItemHeight =
            widget.element.rowDefinitions![e.cellPostion.row!].height ?? 0;
        if (e.rowHeightScale == _rowHeightScale) return;
        _rowHeightScale = e.rowHeightScale;
        WidgetsBinding.instance.addPostFrameCallback((_) {
          setState(() {
            _rowDefinitions[e.cellPostion.row!].height =
                currentItemHeight * e.rowHeightScale;
          });
        });
      } catch (e) {
        print(e);
      }
    }
  }
}

class Element {
  int row;
  int column;

  Element(this.row, this.column);
}
