import 'package:fis_lib_report/converts/event_type.dart';
import 'package:fis_lib_report/report/input_image_list.dart';
import 'package:fis_lib_report/report_info/block_element_info_interface.dart';
import 'package:fis_lib_report/report_info/element_info.dart';
import 'package:fis_lib_report/report_info/report_info.dart';

class InputImageListInfo extends ElementInfo implements IBlockElementInfo {
  bool _isSelected = false;

  ///选中图像选择框时事件通知
  FEventHandler<bool> onSelectedInputChange = FEventHandler<bool>();

  ///列数
  int? column;

  ///是否有缝隙
  bool? hasGap;

  ///是否只读
  bool? isReadOnly;

  ///当前是否处于选中状态
  bool get isSelected => _isSelected;
  set isSelected(bool v) {
    if (_isSelected != v) {
      _isSelected = v;
      onSelectedInputChange.emit(this, v);
    }
  }

  ///已选图像集合
  List<String> get selectedImages => _selectedImages;
  set selectedImages(List<String> v) {
    _selectedImages = v;
    onSelectedChange.emit(this, v);
  }

  ///选中项改变通知UI变化
  FEventHandler<List<String>> onSelectedChange = FEventHandler<List<String>>();

  List<String> _selectedImages = [];

  //图像选中时通知模板
  FEventHandler<String> onSelect = FEventHandler();

  ///图像移除事件
  FEventHandler<String> onRemove = FEventHandler();

  InputImageListInfo.fromElement(InputImageList inputImageList)
      : super.fromElement(inputImageList) {
    column = inputImageList.column;
    hasGap = inputImageList.hasGap;
    isReadOnly = inputImageList.isReadOnly;
    FReportInfo.instance.setMockDatas
        .removeListener((sender, e) => showMockData());
    FReportInfo.instance.setMockDatas
        .addListener((sender, e) => showMockData());
  }

  @override
  List<ElementInfo>? elementInfos;

  ///添加图像
  void addImage(String imageUrl) {
    onSelect.emit(this, imageUrl);
  }

  ///显示MOck数据
  showMockData() {
    selectedImages = [
      'http://cdn-bj.fis.plus/db38b8d36a7b48ada3effe0a53ea8993.jpg',
      'http://cdn-bj.fis.plus/db38b8d36a7b48ada3effe0a53ea8993.jpg'
    ];
  }

  @override
  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['Key'] = id;
    map['Value'] = selectedImages;
    map['Name'] = tag == null ? '' : tag!.name;
    map['TagId'] = tag == null ? '' : tag!.id;
    FReportInfo.instance.jsonItems.add(map);
    return map;
  }
}
