import 'package:fis_lib_report/converts/event_type.dart';
import 'package:fis_lib_report/report/input_text.dart';
import 'package:fis_lib_report/report_info/element_tag_names.dart';
import 'package:fis_lib_report/report_info/report_info.dart';
import 'package:fis_lib_report/report_info/text_element_info.dart';

class InputTextInfo extends TextElementInfo {
  ///是否只读
  bool? isReadOnly;

  ///输入框宽度
  double lineWidth = 0.0;

  ///当前输入框是否已经监听
  bool? isListening;

  ///UI绑定的文本
  String get text => _text;

  set text(String v) {
    _text = v;
    onTextChange.emit(this, v);
  }

  set writeOnly(String v) {
    _text = v;
  }

  ///语音输入or测量结果标志
  late List<String> featureTag;

  ///文本改变通知UI变化
  FEventHandler<String> onTextChange = FEventHandler<String>();

  String _text = '';

  InputTextInfo.fromElement(InputText element) : super.fromElement(element) {
    isReadOnly = element.isReadOnly;
    isListening = false;
    text = '';
    featureTag = element.featureTag;
    FReportInfo.instance.setMockDatas
        .removeListener((sender, e) => showMockData());
    FReportInfo.instance.setMockDatas
        .addListener((sender, e) => showMockData());
  }

  @override
  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['Key'] = id;
    map['Value'] = text;
    map['Name'] = tag == null ? '' : tag!.name;
    map['TagId'] = tag == null ? '' : tag!.id;
    FReportInfo.instance.jsonItems.add(map);
    return map;
  }

  ///显示MOck数据
  showMockData() {
    try {
      final tagName = tag!.name;
      if (tagName == TagNames.HOSPITALNAME) {
        text = 'XXXX医院';
      } else if (tagName == TagNames.REPORTDES ||
          tagName == TagNames.REPORTSUMARY) {
        text = 'XX\r\nXX\r\nXX\r\nXX\r\nXX';
      } else {
        text = 'XXX';
      }
    } catch (e) {
      print(e);
    }
  }
}
