import 'package:fis_lib_report/converts/event_type.dart';
import 'package:fis_lib_report/report/element_type.dart';
import 'package:fis_lib_report/report/interfaces/block_element.dart';
import 'package:fis_lib_report/report/interfaces/element.dart';
import 'package:fis_lib_report/report/rt_table.dart';
import 'package:fis_lib_report/report_info/block_element_info_interface.dart';
import 'package:fis_lib_report/report_info/date_time_info.dart';
import 'package:fis_lib_report/report_info/element_info.dart';
import 'package:fis_lib_report/report_info/element_tag_names.dart';
import 'package:fis_lib_report/report_info/input_image_info.dart';
import 'package:fis_lib_report/report_info/input_image_list_info.dart';
import 'package:fis_lib_report/report_info/input_text_info.dart';
import 'package:fis_lib_report/report_info/multi_selected_info.dart';
import 'package:fis_lib_report/report_info/multi_selected_vertical_info.dart';
import 'package:fis_lib_report/report_info/paragraph_info.dart';
import 'package:fis_lib_report/report_info/report_base_info.dart';
import 'package:fis_lib_report/report_info/report_event_args.dart';
import 'package:fis_lib_report/report_info/rt_table_info.dart';
import 'package:fis_lib_report/report_info/single_selected_info.dart';

class FReportInfo extends ReportBaseInfo {
  static FReportInfo? _reportInfo;

  FReportInfo._internal();

  ///ReportInfo的key-value的数据集合
  List<dynamic> jsonItems = [];

  ///选择词条按钮点击时触发
  FEventHandler<String> onDiagnosticTap = FEventHandler();

  ///语音输入
  FEventHandler<InputTextInfo> onSpeechInput = FEventHandler();

  ///设置默认数据，用于UI展示
  FEventHandler<String> setMockDatas = FEventHandler();

  bool isASROn = true;

  ///ReportInfo全局单例
  static FReportInfo get instance {
    _reportInfo ??= FReportInfo._internal();
    return _reportInfo!;
  }

  ///从Json中加载数据
  void fromJson(List<dynamic> json) {
    for (var item in json) {
      final itemJson = item as Map<String, dynamic>;
      final key = itemJson['Key'];
      final value = itemJson['Value'] ?? '';
      final elementInfo = getElementInfoById(key);
      if (elementInfo != null) {
        final elementType = elementInfo.elementType;
        if (elementType!.name == ElementType.inputText.name) {
          final inputInfo = elementInfo as InputTextInfo;
          inputInfo.text = value;
        } else if (elementType.name == ElementType.singleSelected.name) {
          final info = elementInfo as SingleSelectedInfo;
          info.selectedItem = value;
        } else if (elementType.name == ElementType.multiSelected.name) {
          final info = elementInfo as MulitiSelectedInfo;
          List<String> list = [];
          final items = value as List<dynamic>;
          for (var i in items) {
            list.add(i.toString());
          }
          info.selectedItems = list;
        } else if (elementType.name == ElementType.multiSelectedVertical.name) {
          final info = elementInfo as MulitiSelectedVerticalInfo;
          List<String> list = [];
          final items = value as List<dynamic>;
          for (var i in items) {
            list.add(i.toString());
          }
          info.selectedItems = list;
        } else if (elementType.name == ElementType.imageList.name) {
          final imagesInfo = getElementInfoById(key) as InputImageListInfo;
          final images = value as List<dynamic>;
          List<String> targetImages = [];
          for (var i in images) {
            targetImages.add(i.toString());
          }
          imagesInfo.selectedImages = targetImages;
        } else if (elementType.name == ElementType.dateTime.name) {
          final info = elementInfo as DateTimeInfo;
          info.text = value;
        } else if (elementType.name == ElementType.inputImage.name) {
          final info = elementInfo as InputImageInfo;
          info.selectedImage = value;
        }
      }
    }
  }

  ///插入报告描述&超声提示
  void insertDiagnostic(String desc, String tips) {
    try {
      try {
        if (desc.isNotEmpty) {
          final descInfo =
              getElementInfoByTagName(TagNames.REPORTDES) as InputTextInfo;
          if (descInfo.text.isEmpty) {
            descInfo.text = desc;
          } else {
            descInfo.text = descInfo.text + '\r\n' + desc;
          }
        }
      } catch (ex) {
        print(ex);
      }
      if (tips.isNotEmpty) {
        final tipsInfo =
            getElementInfoByTagName(TagNames.REPORTSUMARY) as InputTextInfo;
        if (tipsInfo.text.isEmpty) {
          tipsInfo.text = tips;
        } else {
          tipsInfo.text = tipsInfo.text + '\r\n' + tips;
        }
      }
    } catch (e) {
      print(e);
    }
  }

  ///ReportInfo转为Json,用于报告存储至Server
  Map<String, dynamic> toJson() {
    jsonItems.clear();
    final map = <String, dynamic>{};
    try {
      final headInfos = [];
      for (var element in headers) {
        headInfos.add(_getElementInfo(element));
      }
      map['HeadInfos'] = headInfos;
      final blockInfos = [];
      for (var block in blocks) {
        blockInfos.add(_getElementInfo(block));
      }
      map['BlockInfos'] = blockInfos;
      final footerInfos = [];
      for (var block in footers) {
        footerInfos.add(_getElementInfo(block));
      }
      map['FooterInfos'] = footerInfos;
    } catch (e) {
      print(e);
    }
    return map;
  }

  ///重新渲染UI&重构reportInfo
  void reload(
    String reporter,
    DateTime reportDate,
    String jsonStr,
    FEventHandler<String> onSelect,
    String revoke,
    String selectEntry,
    String selectImageHint, {
    bool isASROn = true,
  }) {
    _reportInfo = null;

    this.revoke = revoke;
    this.selectEntry = selectEntry;
    this.selectImageHint = selectImageHint;
    onReload.emit(
        this,
        ReportEventArgs(reportDate, reporter, jsonStr, onSelect,
            isASROn: isASROn));
  }

  ///获取二级ElementInfo(布局组件，非最基础的组件)
  IBlockElementInfo? getBlockElement(IBlockElement block) {
    final id = block.id;
    for (var element in headers) {
      if (element.id == id) {
        return element;
      }
    }
    for (var element in blocks) {
      if (element.id == id) {
        return element;
      }
    }
    for (var element in footers) {
      if (element.id == id) {
        return element;
      }
    }
    return null;
  }

  ///获取报告模板中当前处于选中状态的图像输入框
  void selectedInputImage(String imageUrl) {
    for (var element in headers) {
      if (_checkImageisSelected(element, imageUrl)) {
        return;
      }
    }
    for (var element in blocks) {
      if (_checkImageisSelected(element, imageUrl)) {
        return;
      }
    }
    for (var element in footers) {
      if (_checkImageisSelected(element, imageUrl)) {
        return;
      }
    }
  }

  ///跟据UI组件获取ReportInfo中的组件
  ElementInfo? getElementInfo(IElement element) {
    try {
      for (var e in headers) {
        final elementInfo = _getBaseElementInfo(e, element);
        if (elementInfo != null) {
          return elementInfo;
        }
      }
      for (var e in blocks) {
        final elementInfo = _getBaseElementInfo(e, element);
        if (elementInfo != null) {
          return elementInfo;
        }
      }
      for (var e in footers) {
        final elementInfo = _getBaseElementInfo(e, element);
        if (elementInfo != null) {
          return elementInfo;
        }
      }
    } catch (e) {
      print(e);
    }
    return null;
  }

  bool _checkImageisSelected(IBlockElementInfo block, String url) {
    if (block is InputImageListInfo) {
      final inputImage = block;
      if (inputImage.isSelected) {
        inputImage.addImage(url);
        return true;
      }
    } else if (block is ParagraphInfo) {
      for (var el in block.elementInfos!) {
        if (el is InputImageInfo) {
          final image = el;
          if (image.isSelected!) {
            image.addImage(url);
            return true;
          }
        }
      }
    } else if (block is RTTableInfo) {
      final cells = block.cells ?? {};
      for (var cell in cells.values) {
        for (var b in cell.blocks ?? []) {
          if (b.elementType?.name == ElementType.imageList.name) {
            final imageList = b as InputImageListInfo;
            if (imageList.isSelected) {
              imageList.addImage(url);
              return true;
            }
          }
        }
      }
    }
    return false;
  }

  ///获取基础组件信息
  ElementInfo? _getBaseElementInfo(IBlockElementInfo block, IElement element) {
    ElementInfo? result;
    final type = block.elementType;
    if (type!.name == ElementType.paragraph.name) {
      final paragraph = block as ParagraphInfo;
      for (var el in paragraph.elementInfos!) {
        if (el.id == element.id) {
          result = el;
          break;
        }
      }
    }
    if (result == null) {
      if (type.name == ElementType.rtTable.name) {
        final table = block as RTTableInfo;
        final cells = table.cells!;
        for (var cell in cells.values) {
          for (var b in cell.blocks!) {
            final info = _getBaseElementInfo(b, element);
            if (info != null) {
              result = info;
              break;
            }
          }
        }
      }
    }

    return result;
  }

  Map<String, dynamic> _getElementInfo(IBlockElementInfo element) {
    final _type = element.elementType!;
    if (_type.name == ElementType.rtTable.name) {
      final table = element as RTTableInfo;
      return table.toJson();
    } else if (_type.name == ElementType.paragraph.name) {
      final paragraph = element as ParagraphInfo;
      return paragraph.toJson();
    } else {
      final inputImageList = element as InputImageListInfo;
      return inputImageList.toJson();
    }
  }
}
