import 'package:fis_lib_report/report/element_type.dart';
import 'package:fis_lib_report/report/input_image_list.dart';
import 'package:fis_lib_report/report/interfaces/cell.dart';
import 'package:fis_lib_report/report/paragraph.dart';
import 'package:fis_lib_report/report/rt_table.dart';
import 'package:fis_lib_report/report_info/block_element_info_interface.dart';
import 'package:fis_lib_report/report_info/element_info.dart';
import 'package:fis_lib_report/report_info/paragraph_info.dart';

import 'input_image_list_info.dart';
import 'rt_table_info.dart';

class RTCellInfo extends ElementInfo {
  List<IBlockElementInfo>? blocks = [];

  RTCellInfo.fromElement(ICell value) : super.fromElement(value) {
    for (var element in value.blocks!) {
      if (element.elementType!.name == ElementType.paragraph.name) {
        final paragraphInfo = element as Paragraph;
        final block = ParagraphInfo.fromElement(paragraphInfo);
        blocks!.add(block);
      } else if (element.elementType!.name == ElementType.imageList.name) {
        final inputImageList = element as InputImageList;
        final block = InputImageListInfo.fromElement(inputImageList);
        blocks!.add(block);
      } else if (element.elementType!.name == ElementType.rtTable.name) {
        final rtTable = element as RTTable;
        final block = RTTableInfo.fromElement(rtTable);
        blocks!.add(block);
      } else {
        print('报告库解析Element失败：${element.elementType?.name}');
      }
    }
  }

  @override
  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map.addAll(super.toJson());
    final blockInfos = <String, dynamic>{};
    for (var b in blocks!) {
      if (b.elementType!.name == ElementType.paragraph.name) {
        final paragraphInfo = b as ParagraphInfo;
        blockInfos.addAll(paragraphInfo.toJson());
      } else if (b.elementType!.name == ElementType.imageList.name) {
        final imageList = b as InputImageListInfo;
        blockInfos.addAll(imageList.toJson());
      } else if (b.elementType!.name == ElementType.rtTable.name) {
        final rtTable = b as RTTableInfo;
        blockInfos.addAll(rtTable.toJson());
      } else {
        print("报告库解析toJson失败${b.elementType!.name}");
      }
    }
    map['Blocks'] = blockInfos;
    return map;
  }
}
