import 'package:fis_lib_report/converts/event_type.dart';
import 'package:fis_lib_report/report/single_selected.dart';
import 'package:fis_lib_report/report_info/report_info.dart';
import 'package:fis_lib_report/report_info/text_element_info.dart';

class SingleSelectedInfo extends TextElementInfo {
  ///是否只读
  bool? isReadOnly;

  ///所有可选项
  List<String>? items = [];

  ///已选中的项
  String get selectedItem => _selectedItem;
  set selectedItem(String v) {
    if (items!.contains(v)) _selectedItem = v;
    onSelectedChange.emit(this, v);
  }

  ///选中项改变通知UI变化
  FEventHandler<String> onSelectedChange = FEventHandler<String>();

  String _selectedItem = '';

  ///UI渲染时获取待选项数据
  SingleSelectedInfo.fromElement(SingleSelected element)
      : super.fromElement(element) {
    isReadOnly = element.isReadOnly;
    items = element.items;
    FReportInfo.instance.setMockDatas
        .removeListener((sender, e) => _showMockData());
    FReportInfo.instance.setMockDatas
        .addListener((sender, e) => _showMockData());
  }

  @override
  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['Key'] = id;
    map['Value'] = selectedItem;
    map['Name'] = tag == null ? '' : tag!.name;
    map['TagId'] = tag == null ? '' : tag!.id;
    FReportInfo.instance.jsonItems.add(map);
    return map;
  }

  ///显示Mock数据
  void _showMockData() {
    try {
      if (items!.isNotEmpty) {
        selectedItem = items!.first;
      } else {
        items!.add('XX');
        selectedItem = "XX";
      }
    } catch (e) {
      print(e);
    }
  }
}
